from flask import current_app

from typing import List, Optional
from mypy_extensions import TypedDict

from .connector import Connector
from .decorators import checks_policy_property
from .storage_gateway import StorageGateway
from globus.manager.constants import ConfigurationObjectTag
import json


EnvironmentSettingType = TypedDict(
    "EnvironmentSettingType", {"name": str, "value": str}
)


class IrodsStorageGateway(StorageGateway):
    CONNECTOR_ID = "e47b6920-ff57-11ea-8aaa-000c297ab3c2"
    POLICY_DATA_TYPE = "irods_storage_policies#1.0.0"
    POLICY_DATA_TYPES = [POLICY_DATA_TYPE]
    __mapper_args__ = {
        "polymorphic_identity": (
            ConfigurationObjectTag.STORAGE_GATEWAY.name + CONNECTOR_ID
        )
    }

    connector_policy_names = [
        "irods_environment_file",
        "irods_authentication_file",
        "environment",
    ]
    _reserved_envs = [
        "IRODS_ENVIRONMENT_FILE",
        "IRODS_AUTHENTICATION_FILE",
        "irodsConnectAsAdmin",
    ]
    _required_irods_settings = [
        "irods_zone_name",
        "irods_user_name",
        "irods_host",
        "irods_port",
        "irods_default_resource",
    ]

    def __init__(self, *args, **kwargs):
        self._irods_environment_file = None  # type: str
        self._irods_authentication_file = None  # type: str
        self._environment = None  # type: Optional[List[EnvironmentSettingType]]

        super().__init__(*args, **kwargs)

    connector = Connector(id=CONNECTOR_ID, display_name="iRODS")

    @property
    def irods_environment_file(self) -> str:
        if self._irods_environment_file is None:
            raise AttributeError(
                "The irods_environment_file property is not set on this storage gateway."
            )
        return self._irods_environment_file

    @irods_environment_file.setter
    def irods_environment_file(self, value: str):
        self._irods_environment_file = value

    @checks_policy_property(prop="irods_environment_file")
    def _check_irods_environment_file(self, value: str):
        if not value:
            raise ValueError(
                "The irods_environment_file property is required for this connector."
            )
        irods_data = self.get_irods_environment_info(env_file=value)
        if not all(k in irods_data for k in self._required_irods_settings):
            raise ValueError(
                "The irods_environment_file does not contain the required "
                "settings.  The following settings are required: {}.".format(
                    ", ".join(self._required_irods_settings)
                )
            )

    @property
    def irods_authentication_file(self) -> str:
        if self._irods_authentication_file is None:
            raise AttributeError(
                "The irods_authentication_file property is not set on this storage gateway."
            )
        return self._irods_authentication_file

    @irods_authentication_file.setter
    def irods_authentication_file(self, value: str):
        self._irods_authentication_file = value

    @checks_policy_property(prop="irods_authentication_file")
    def _check_irods_authentication_file(self, value: str):
        if not value:
            raise ValueError(
                "The irods_authentication_file property is required for this connector."
            )

    @property
    def environment(self) -> List[EnvironmentSettingType]:
        _env = []  # type: List[EnvironmentSettingType]
        if self._irods_environment_file:
            _env.append(
                {
                    "name": "IRODS_ENVIRONMENT_FILE",
                    "value": self._irods_environment_file,
                }
            )
            _env.append(
                {
                    "name": "irodsConnectAsAdmin",
                    "value": self.get_irods_environment_info()["irods_user_name"],
                }
            )
        if self._irods_authentication_file:
            _env.append(
                {
                    "name": "IRODS_AUTHENTICATION_FILE",
                    "value": self._irods_authentication_file,
                }
            )
        if self._environment:
            for entry in self._environment:
                if entry["name"] not in self._reserved_envs:
                    _env.append(entry)

        return _env

    @environment.setter
    def environment(self, value: Optional[List[EnvironmentSettingType]]) -> None:
        self._environment = value

    @environment.deleter
    def environment(self) -> None:
        self._environment = None

    @checks_policy_property(prop="environment")
    def _check_environment(self, value: Optional[List[EnvironmentSettingType]]):
        if value:
            for entry in value:
                if "name" not in entry:
                    raise ValueError(
                        "Missing environment variable name in {}".format(entry)
                    )
                elif "value" not in entry:
                    raise ValueError(
                        "Missing environment variable value in {}".format(entry)
                    )

    def get_irods_environment_info(self, env_file: Optional[str] = None):
        """
        Read irods_environment_file into a dict.
        """
        if env_file is None:
            env_file = self._irods_environment_file

        try:
            with open(env_file) as f:
                irods_data = json.load(f)
        except IOError as e:
            current_app.logger.error(
                "Error reading irods_environment_file {}: {}".format(env_file, str(e))
            )
            raise ValueError("Unable to access irods_environment_file")
        except json.decoder.JSONDecodeError as e:
            current_app.logger.error(
                "Error parsing irods_environment_file {}: {}".format(env_file, str(e))
            )
            raise ValueError("Unable to parse irods_environment_file")
        except Exception as e:
            current_app.logger.error(
                "Error reading irods_environment_file {}: {}".format(env_file, str(e))
            )
            raise ValueError("Invalid irods_environment_file")
        return irods_data

    def home_directory(self, username):
        zone = self.get_irods_environment_info()["irods_zone_name"]
        return "/{}/home/{}".format(zone, username)

    def get_user_display_name(self, username):
        return username

    def default_mapping(self, identity_data):
        if len(self.allowed_domains) == 1:
            user, domain = identity_data["username"].rsplit("@", 1)
            if domain in self.allowed_domains:
                return user

    @property
    def load_dsi_module(self):
        return "iRODS"

    @load_dsi_module.setter
    def load_dsi_module(self, value: str):
        pass

    @property
    def process_user(self):
        return "gcsweb"

    @process_user.setter
    def process_user(self, value):
        pass

    def __repr__(self):
        return "<IrodsStorageGateway {}>".format(self.display_name)
