from typing import List, Optional

from mypy_extensions import TypedDict

from .connector import Connector
from .decorators import checks_policy_property
from .posix_storage_gateway import PosixStorageGateway
from globus.manager.constants import ConfigurationObjectTag


EnvironmentSettingType = TypedDict(
    "EnvironmentSettingType", {"name": str, "value": str}
)


class PosixStagingStorageGateway(PosixStorageGateway):
    CONNECTOR_ID = "052be037-7dda-4d20-b163-3077314dc3e6"
    POLICY_DATA_TYPE = "posix_staging_storage_policies#1.0.0"
    POLICY_DATA_TYPES = [POLICY_DATA_TYPE]

    __mapper_args__ = {
        "polymorphic_identity": (
            ConfigurationObjectTag.STORAGE_GATEWAY.name + CONNECTOR_ID
        )
    }

    connector_policy_names = PosixStorageGateway.connector_policy_names + [
        "stage_app",
        "environment",
    ]  # type: List[str]

    def __init__(self, *args, **kwargs):
        self._stage_app = None  # type: str
        self._environment = None  # type: Optional[List[EnvironmentSettingType]]

        super().__init__(*args, **kwargs)

    connector = Connector(id=CONNECTOR_ID, display_name="POSIX Staging")

    @property
    def stage_app(self) -> str:
        if self._stage_app is None:
            raise AttributeError(
                "The stage_app property is not set on this storage gateway."
            )
        return self._stage_app

    @stage_app.setter
    def stage_app(self, value: str):
        self._stage_app = value

    @checks_policy_property(prop="stage_app")
    def _check_stage_app(self, value: str):
        if not value:
            raise ValueError("The stage_app property is required for this connector.")

    @property
    def environment(self) -> List[EnvironmentSettingType]:
        if self._environment is None:
            raise AttributeError(
                "The environment property is not set on this storage gateway."
            )
        return self._environment

    @environment.setter
    def environment(self, value: Optional[List[EnvironmentSettingType]]) -> None:
        self._environment = value

    @environment.deleter
    def environment(self) -> None:
        self._environment = None

    @checks_policy_property(prop="environment")
    def _check_environment(self, value: Optional[List[EnvironmentSettingType]]):
        if value:
            for entry in value:
                if "name" not in entry:
                    raise ValueError(
                        "Missing environment variable name in {}".format(entry)
                    )
                elif "value" not in entry:
                    raise ValueError(
                        "Missing environment variable value in {}".format(entry)
                    )

    def __repr__(self):
        return "<PosixStagingStorageGateway {}>".format(self.display_name)
